package com.sun.jmx.snmp;

import java.util.logging.Level;
import com.sun.jmx.defaults.JmxProperties;

public class SnmpMessage extends SnmpMsg implements SnmpDefinitions
{
    public byte[] community;
    
    public SnmpMessage() {
    }
    
    @Override
    public int encodeMessage(byte[] array) throws SnmpTooBigException {
        if (this.data == null) {
            throw new IllegalArgumentException("Data field is null");
        }
        int trim;
        try {
            BerEncoder berEncoder = new BerEncoder(array);
            berEncoder.openSequence();
            berEncoder.putAny(this.data, this.dataLength);
            berEncoder.putOctetString((this.community != null) ? this.community : new byte[0]);
            berEncoder.putInteger(this.version);
            berEncoder.closeSequence();
            trim = berEncoder.trim();
        }
        catch (ArrayIndexOutOfBoundsException ex) {
            throw new SnmpTooBigException();
        }
        return trim;
    }
    
    @Override
    public int getRequestId(byte[] array) throws SnmpStatusException {
        BerDecoder berDecoder;
        BerDecoder berDecoder2;
        int fetchInteger;
        try {
            berDecoder = new BerDecoder(array);
            berDecoder.openSequence();
            berDecoder.fetchInteger();
            berDecoder.fetchOctetString();
            berDecoder2 = new BerDecoder(berDecoder.fetchAny());
            berDecoder2.openSequence(berDecoder2.getTag());
            fetchInteger = berDecoder2.fetchInteger();
        }
        catch (BerException ex) {
            throw new SnmpStatusException("Invalid encoding");
        }
        try {
            berDecoder.closeSequence();
        }
        catch (BerException ex2) {}
        try {
            berDecoder2.closeSequence();
        }
        catch (BerException ex3) {}
        return fetchInteger;
    }
    
    @Override
    public void decodeMessage(byte[] array, int n) throws SnmpStatusException {
        try {
            BerDecoder berDecoder = new BerDecoder(array);
            berDecoder.openSequence();
            this.version = berDecoder.fetchInteger();
            this.community = berDecoder.fetchOctetString();
            this.data = berDecoder.fetchAny();
            this.dataLength = this.data.length;
            berDecoder.closeSequence();
        }
        catch (BerException ex) {
            throw new SnmpStatusException("Invalid encoding");
        }
    }
    
    @Override
    public void encodeSnmpPdu(SnmpPdu snmpPdu, int n) throws SnmpStatusException, SnmpTooBigException {
        SnmpPduPacket snmpPduPacket = (SnmpPduPacket)snmpPdu;
        this.version = snmpPduPacket.version;
        this.community = snmpPduPacket.community;
        this.address = snmpPduPacket.address;
        this.port = snmpPduPacket.port;
        this.data = new byte[n];
        try {
            BerEncoder berEncoder = new BerEncoder(this.data);
            berEncoder.openSequence();
            this.encodeVarBindList(berEncoder, snmpPduPacket.varBindList);
            switch (snmpPduPacket.type) {
                case 160:
                case 161:
                case 162:
                case 163:
                case 166:
                case 167:
                case 168: {
                    SnmpPduRequest snmpPduRequest = (SnmpPduRequest)snmpPduPacket;
                    berEncoder.putInteger(snmpPduRequest.errorIndex);
                    berEncoder.putInteger(snmpPduRequest.errorStatus);
                    berEncoder.putInteger(snmpPduRequest.requestId);
                    break;
                }
                case 165: {
                    SnmpPduBulk snmpPduBulk = (SnmpPduBulk)snmpPduPacket;
                    berEncoder.putInteger(snmpPduBulk.maxRepetitions);
                    berEncoder.putInteger(snmpPduBulk.nonRepeaters);
                    berEncoder.putInteger(snmpPduBulk.requestId);
                    break;
                }
                case 164: {
                    SnmpPduTrap snmpPduTrap = (SnmpPduTrap)snmpPduPacket;
                    berEncoder.putInteger(snmpPduTrap.timeStamp, 67);
                    berEncoder.putInteger(snmpPduTrap.specificTrap);
                    berEncoder.putInteger(snmpPduTrap.genericTrap);
                    if (snmpPduTrap.agentAddr != null) {
                        berEncoder.putOctetString(snmpPduTrap.agentAddr.byteValue(), 64);
                    }
                    else {
                        berEncoder.putOctetString(new byte[0], 64);
                    }
                    berEncoder.putOid(snmpPduTrap.enterprise.longValue());
                    break;
                }
                default: {
                    throw new SnmpStatusException("Invalid pdu type " + String.valueOf(snmpPduPacket.type));
                }
            }
            berEncoder.closeSequence(snmpPduPacket.type);
            this.dataLength = berEncoder.trim();
        }
        catch (ArrayIndexOutOfBoundsException ex) {
            throw new SnmpTooBigException();
        }
    }
    
    @Override
    public SnmpPdu decodeSnmpPdu() throws SnmpStatusException {
        BerDecoder berDecoder = new BerDecoder(this.data);
        SnmpPduRequest snmpPduRequest2;
        try {
            int tag = berDecoder.getTag();
            berDecoder.openSequence(tag);
            switch (tag) {
                case 160:
                case 161:
                case 162:
                case 163:
                case 166:
                case 167:
                case 168: {
                    SnmpPduRequest snmpPduRequest = new SnmpPduRequest();
                    snmpPduRequest.requestId = berDecoder.fetchInteger();
                    snmpPduRequest.errorStatus = berDecoder.fetchInteger();
                    snmpPduRequest.errorIndex = berDecoder.fetchInteger();
                    snmpPduRequest2 = snmpPduRequest;
                    break;
                }
                case 165: {
                    SnmpPduBulk snmpPduBulk = new SnmpPduBulk();
                    snmpPduBulk.requestId = berDecoder.fetchInteger();
                    snmpPduBulk.nonRepeaters = berDecoder.fetchInteger();
                    snmpPduBulk.maxRepetitions = berDecoder.fetchInteger();
                    snmpPduRequest2 = snmpPduBulk;
                    break;
                }
                case 164: {
                    SnmpPduTrap snmpPduTrap = new SnmpPduTrap();
                    snmpPduTrap.enterprise = new SnmpOid(berDecoder.fetchOid());
                    byte[] fetchOctetString = berDecoder.fetchOctetString(64);
                    if (fetchOctetString.length != 0) {
                        snmpPduTrap.agentAddr = new SnmpIpAddress(fetchOctetString);
                    }
                    else {
                        snmpPduTrap.agentAddr = null;
                    }
                    snmpPduTrap.genericTrap = berDecoder.fetchInteger();
                    snmpPduTrap.specificTrap = berDecoder.fetchInteger();
                    snmpPduTrap.timeStamp = (long)berDecoder.fetchInteger(67);
                    snmpPduRequest2 = snmpPduTrap;
                    break;
                }
                default: {
                    throw new SnmpStatusException(9);
                }
            }
            snmpPduRequest2.type = tag;
            snmpPduRequest2.varBindList = this.decodeVarBindList(berDecoder);
            berDecoder.closeSequence();
        }
        catch (BerException thrown) {
            if (JmxProperties.SNMP_LOGGER.isLoggable(Level.FINEST)) {
                JmxProperties.SNMP_LOGGER.logp(Level.FINEST, SnmpMessage.class.getName(), "decodeSnmpPdu", "BerException", thrown);
            }
            throw new SnmpStatusException(9);
        }
        catch (IllegalArgumentException thrown2) {
            if (JmxProperties.SNMP_LOGGER.isLoggable(Level.FINEST)) {
                JmxProperties.SNMP_LOGGER.logp(Level.FINEST, SnmpMessage.class.getName(), "decodeSnmpPdu", "IllegalArgumentException", thrown2);
            }
            throw new SnmpStatusException(9);
        }
        snmpPduRequest2.version = this.version;
        snmpPduRequest2.community = this.community;
        snmpPduRequest2.address = this.address;
        snmpPduRequest2.port = this.port;
        return snmpPduRequest2;
    }
    
    @Override
    public String printMessage() {
        StringBuffer sb = new StringBuffer();
        if (this.community == null) {
            sb.append("Community: null");
        }
        else {
            sb.append("Community: {\n");
            sb.append(SnmpMsg.dumpHexBuffer(this.community, 0, this.community.length));
            sb.append("\n}\n");
        }
        return sb.append(super.printMessage()).toString();
    }
}
